#ifndef GEOMETRIC_FIGURES_BASE
#define GEOMETRIC_FIGURES_BASE

#include "graphlib.h"

// colors of the shapes
enum DrawingColor
{
   white,  blue,   teal,      green,     turquoise, darkgray,
   brown,  purple, lightblue, lightgray, gold,      red,
   orange, pink,   yellow,    black
};

// wrapper class for the colors
class ShapeColor
{
   private:
      DrawingColor color;

      // table defining conversion from simple colors
      // to necessary Windows definitions
      static const ezdColor g_aColors[];

   public:
      // constructor
      ShapeColor(DrawingColor c = blue);

      // increment and decrement operators
      ShapeColor operator++ (int);
      ShapeColor& operator++ ();
      ShapeColor operator-- (int);
      ShapeColor& operator-- ();

      // relational operators
      friend bool operator== (const ShapeColor& lhs,
                          const ShapeColor& rhs);
      friend bool operator!= (const ShapeColor& lhs,
                          const ShapeColor& rhs);
      friend bool operator< (const ShapeColor& lhs,
                          const ShapeColor& rhs);
      friend bool operator<= (const ShapeColor& lhs,
                          const ShapeColor& rhs);
      friend bool operator> (const ShapeColor& lhs,
                          const ShapeColor& rhs);
      friend bool operator>= (const ShapeColor& lhs,
                          const ShapeColor& rhs);

      // convert to Windows definition for colors
      ezdColor convertToEzdColor();
};

// ***********************************************************
//      ShapeColor class implementation
// ***********************************************************

// color conversion lookup table
const ezdColor ShapeColor::g_aColors[] = {
   ezdWhite,
   ezdBlue,
   ezdTeal,
   ezdGreen,
   ezdTurquoise,
   ezdDarkGray,
   ezdBrown,
   ezdPurple,
   ezdLightBlue,
   ezdLightGray,
   ezdGold,
   ezdRed,
   ezdOrange,
   ezdPink,
   ezdYellow,
   ezdBlack
};

// constructor. initialize color
ShapeColor::ShapeColor(DrawingColor c): color(c)
{}

// postfix increment
ShapeColor ShapeColor::operator++ (int)
{
   DrawingColor origColor = color;

   color = DrawingColor((color+1) % 16);
   return ShapeColor(origColor);
}

// prefix increment
ShapeColor& ShapeColor::operator++ ()
{
   color = DrawingColor((color+1) % 16);
   return *this;
}

// postfix decrement
ShapeColor ShapeColor::operator-- (int)
{
   DrawingColor origColor = color;

   if (color == black)
      color = white;
   else
      color = DrawingColor(color-1);
   return ShapeColor(origColor);
}

// prefix decrement
ShapeColor& ShapeColor::operator-- ()
{
   if (color == black)
      color = white;
   else
      color = DrawingColor(color-1);
   return *this;
}

// use lookup table to convert to Windows color definitions
ezdColor ShapeColor::convertToEzdColor()
{
   return g_aColors[color];
}

// relational operators
bool operator== (const ShapeColor& lhs,
                 const ShapeColor& rhs)
{
   return lhs.color == rhs.color;
}

bool operator!= (const ShapeColor& lhs,
                 const ShapeColor& rhs)
{
   return lhs.color != rhs.color;
}

bool operator< (const ShapeColor& lhs,
                const ShapeColor& rhs)
{
   return lhs.color < rhs.color;
}

bool operator<= (const ShapeColor& lhs,
                 const ShapeColor& rhs)
{
   return lhs.color <= rhs.color;
}

bool operator> (const ShapeColor& lhs,
                const ShapeColor& rhs)
{
   return lhs.color > rhs.color;
}

bool operator>= (const ShapeColor& lhs,
                 const ShapeColor& rhs)
{
   return lhs.color >= rhs.color;
}

// class point to be used within Shape -- pld

class Point {
private:
	double _x, _y;
public:
	Point() : _x(0), _y(0) {}
	Point(double x, double y) : _x(x), _y(y) {}
	double x() const {return _x;}
	double y() const {return _y;}
};


// a generic class that defines a point, color and
// member functions to access and change these arguments.
// the class is inherited by geometric figure classes that
// implement specific drawing operations
class Shape
{
   private:
     // location of the base point
      //double baseX, baseY;
	  Point _base;	// pld

      // color of the shape
      ShapeColor color;

   private:
     // handle to the shape, used only for deletion
     EZDHANDLE shape_handle;

   public:
      // constructor. requires the base point for drawing
      // and the color to be used when a shape is drawn
      Shape(double x, double y, ShapeColor c);
      // virtual destructor may be necessary for derived class
      virtual ~Shape();

      // functions to access coordinates of base point
      double getX() const;              // return x coordinate
      double getY() const;              // return y coordinate

      // change base point
      void move(double x, double y);

      // member functions to access the color for the shape
      ShapeColor getColor() const;           // return color
      void setColor(ShapeColor c);           // change color

      // draw shape. must be implemented in a derived class
	  virtual void draw() = 0;
      
      // erase the shape
      virtual void erase();

      // update the shape
      virtual void update();

	  // added by pld to allow shape_handle to be private
	  // rather than protected. Note there is no accessor!
	  // shape_handle is used only by Shape::erase().
	  void setHandle(EZDHANDLE sh) {
		  shape_handle = sh;
	  }
};

// ***********************************************************
//      Shape class implementation
// ***********************************************************

// initialize basepoint coordinates and color.
// specify that no shape has been drawn by setting
// shape_handle to 0
Shape::Shape(double x, double y, ShapeColor c):
		  // baseX(x), baseY(y),
		  _base(x,y),
		  color(c), shape_handle(0)
{}

Shape::~Shape() {}

double Shape::getX() const
{
   return _base.x(); //baseX;
}

double Shape::getY() const
{
   return _base.y();
}

ShapeColor Shape::getColor() const
{
   return color;
}

void Shape::setColor(ShapeColor c)
{
   color = c;
}

void Shape::move(double x, double y)
{
  // baseX = x;
  // baseY = y;
	_base = Point(x,y);
}

void Shape::erase()
{
   // shape_handle identifies the shape in the window.
   // the shape is defined if shape_handle is not 0
   if (shape_handle != 0)
      ezdDeleteShape(shape_handle); // erase the shape
   shape_handle = 0;                // no shape defined
}

void Shape::update()
{
   erase();
   draw();
}

// The following point class derived from Shape is not used.
//class SPoint : public Shape {};

// Part of the Spring 2004 comp 272 assignment is to 
// rewrite parts of this file to make greater use of
// the following class. 
// Note that this class Point does *not* derive from Shape
// (life is simpler that way).
// Note also that there are no mutators: If you want to
// change a point, just create a new one:
//	    _p = Point(_p.x() + delta_x, _p.y() + delta_y);
// Or else go ahead and add mutators, if you prefer.

 #endif   // GEOMETRIC_FIGURES_BASE
